/***********************************************************
 * combine.c
 *    Rfer to combine.m for documentation.
 *
 * Author: David Ross, Oct. 2005
 * $Id: combine.c,v 1.1.1.1 2006/06/01 15:32:21 dross Exp $
 ***********************************************************/

#include "assert.h"
#include "mex.h"

void mexFunction(int nlhs, mxArray *plhs[],
                 int nrhs, const mxArray *prhs[]) {
   /* Declare variables. */ 
   double *tensor = NULL;
   const int *tensor_size = NULL;
   double *u = NULL;
   double *result = NULL;
   int I,J,K,i,j,k;

   /* Check for proper number of input and output arguments. */    
   if (nrhs != 2) {
      mexErrMsgTxt("Two input arguments required.");
   } 
   if (nlhs > 1) {
      mexErrMsgTxt("Too many output arguments.");
   }

   /* Check data type of input arguments. */
   if (!(mxIsDouble(prhs[0])) || !(mxIsDouble(prhs[1]))) {
      mexErrMsgTxt("Input arrays must be of type double.");
   }

   /* Make sure the arguments have the correct dimensions */
   if (mxGetNumberOfDimensions(prhs[0]) > 3) {
      mexErrMsgTxt("First argument must be a <= 3-dimensional array");
   }
   if (mxGetNumberOfDimensions(prhs[1]) > 2) {
      mexErrMsgTxt("Second argument must have <= 2 dimensions");
   }

   /* Get the data. */
   tensor = (double *)mxGetPr(prhs[0]);
   tensor_size = mxGetDimensions(prhs[0]);
   I = tensor_size[0]; assert(I > 0);
   J = tensor_size[1]; assert(J > 0);
   if (mxGetNumberOfDimensions(prhs[0]) == 2) {
      K = 1;
   } else {
      K = tensor_size[2]; assert(K > 0);
   }
   u = (double *)mxGetPr(prhs[1]);
   
   /* Make sure u is the right size */
   if (mxGetNumberOfElements(prhs[1]) != K) {
      mexErrMsgTxt("#elements in second arg must match size of 3rd dimension of first arg");
   }
   
   /* Allocate the output variable */
   plhs[0] = mxCreateDoubleMatrix(I,J,mxREAL);
   result = mxGetPr(plhs[0]);

   for(i=0; i<I;i++) {
      for(j=0; j<J; j++) {
	 result[I*j+i] = 0.0;
	 for(k=0; k<K; k++) {
	    result[I*j+i] += tensor[(I*J)*k + I*j + i] * u[k];
	 }
      }
   }
}
